# coding: utf-8

import tushare

class SixQuantData(object):
    def __init__(self):
        pass

    def get_stock_list(self,date):
        pass



sq_data = SixQuantData()


tushare.get_concept_classified()
tushare.get_k_data()